#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "fonts.h"

//-----------------------------------------------------------------------------
// Name: changebmptofont()
// Desc: Change the bitmap to a font
//-----------------------------------------------------------------------------

void FontExtractor::changebmptofont(char *file, char* path){
	char file1[_MAX_PATH];
	char file2[_MAX_PATH];

	strcpy(file1,".\\game\\pictures\\");
	strcat(file1,file);
	strcat(file1,"ch.bmp");

	strcpy(file2,path);
	strcat(file2,"\\");
	strcat(file2,file);
	strcat(file2,".ch");

	if((fp = fopen(file1,"rb"))==NULL){
		return;
	}
	fw = fopen(file2,"wb");
	readheader();
	readbitmap();
	writech();
	fclose(fp);
	fclose(fw);
}

//-----------------------------------------------------------------------------
// Name: changefonttobmp()
// Desc: Change the font to a bitmap
//-----------------------------------------------------------------------------

void FontExtractor::changefonttobmp(char *file, char* path){
	char file1[_MAX_PATH];
	char file2[_MAX_PATH];

	strcpy(file1,path);
	strcat(file1,"\\");
	strcat(file1,file);
	strcat(file1,".ch");

	strcpy(file2,".\\game\\pictures\\");
	strcat(file2,file);
	strcat(file2,"ch.bmp");

	if((fp = fopen(file1,"rb"))==NULL){
		return;
	}
	fw = fopen(file2,"wb");
	writeheader();
	writebitmap();
	drawbitmap();
	writeend();
	fclose(fp);
	fclose(fw);
}

//-----------------------------------------------------------------------------
// Name: drawbitmap()
// Desc: Draw the bitmap, along with the red border
//-----------------------------------------------------------------------------

void FontExtractor::drawbitmap(){
	int num;
	for(int d = 0; d < 145; d++){
		fputc(0x00,fw);
		fputc(0x00,fw);
		fputc(0xff,fw);
	}
	fputc(0x00,fw);
	for(int a = 63; a >= 0; a--){
		for(int b = 0; b < 128; b++){
			if(b%8==0){
				fputc(0x00,fw);
				fputc(0x00,fw);
				fputc(0xff,fw);
			}
			num = pixels[b][a];
			if(num == 0){
				for(int c = 0; c < 3; c++){
					fputc(0x00,fw);
				}
			}
			else{
				for(int c = 0; c < 3; c++){
					fputc(0xff,fw);
				}
			}
			if(b == 127){
				fputc(0x00,fw);
				fputc(0x00,fw);
				fputc(0xff,fw);
				fputc(0x00,fw);
			}
		}
		if(a%8==0){
			for(int d = 0; d < 145; d++){
				fputc(0x00,fw);
				fputc(0x00,fw);
				fputc(0xff,fw);
			}
			fputc(0x00,fw);
		}
	}
}

//-----------------------------------------------------------------------------
// Name: readbitmap()
// Desc: Read the bitmap to convert to *.ch, and skip the unnecessary bytes
//-----------------------------------------------------------------------------

void FontExtractor::readbitmap(){
	int bytes;
	fseek(fp,0x01b4,SEEK_CUR);
	for(int b = 63; b >=0; b--){
		for(int a = 0; a < 128; a++){
			if(a%8==0){
				fseek(fp,0x03,SEEK_CUR);
			}
			fseek(fp,0x02,SEEK_CUR);
			bytes = fgetc(fp);
			if(bytes == 0){
				pixels[a][b] = 0;
			}
			else{
				pixels[a][b] = 1;
			}
		}
		fseek(fp,0x04,SEEK_CUR);
		if(b%8==0){
			fseek(fp,0x01b4,SEEK_CUR);
		}
	}
}

//-----------------------------------------------------------------------------
// Name: readheader()
// Desc: Skip the bitmap header
//-----------------------------------------------------------------------------

void FontExtractor::readheader(){
	fseek(fp,0x36,SEEK_SET);
}

//-----------------------------------------------------------------------------
// Name: writebitmap()
// Desc: Read *.ch and write the corresponding bits to the bitmap
//-----------------------------------------------------------------------------

void FontExtractor::writebitmap(){
	int bytes;
	bool loop = true;
	while(loop){
		for(int b = 0; b < 8; b++){
			for(int a = 0; a < 16; a++){
				for(int num = 0; num < 8; num++){
						bytes = fgetc(fp);
						writebits(bytes,num,a,b);
				}
			}
		}
		break;
	}
}

//-----------------------------------------------------------------------------
// Name: writebits()
// Desc: Write the font character row to the *.ch file
//-----------------------------------------------------------------------------

void FontExtractor::writebits(int num, int a, int b){
	int z;
	int y = 0;
	int q;
	for(int x = 7; x >= 0; x--){
		z = (int)(pow(2,x));
		q=(z*pixels[(a*8)+(7-x)][(b*8)+num]);
		y+=q;
	}
	fputc(y,fw);
}

//-----------------------------------------------------------------------------
// Name: writebits()
// Desc: Write a row of the font character
//-----------------------------------------------------------------------------

void FontExtractor::writebits(int bytes, int num, int a, int b){
	int z;
	int y;
	for(int x = 7; x >= 0; x--){
		z = (int)(pow(2,x));
		y = bytes/z;
		bytes-=(z*y);
		pixels[(a*8)+(7-x)][(b*8)+num] = y;
	}
}

//-----------------------------------------------------------------------------
// Name: writech()
// Desc: ???
//-----------------------------------------------------------------------------

void FontExtractor::writech(){
	for(int b = 0; b < 8; b++){
		for(int a = 0; a < 16; a++){
			for(int num = 0; num < 8; num++){
				writebits(num,a,b);
			}
		}
	}
}

//-----------------------------------------------------------------------------
// Name: writeend()
// Desc: Write the end to the bitmap
//-----------------------------------------------------------------------------

void FontExtractor::writeend(){
	writezeros(2);
}

//-----------------------------------------------------------------------------
// Name: writeheader()
// Desc: Write the header for the bitmap
//-----------------------------------------------------------------------------

void FontExtractor::writeheader(){
	fputc(0x42,fw);
	fputc(0x4d,fw);
	writezeros(8);
	fputc(0x36,fw);
	writezeros(3);
	fputc(0x28,fw);
	writezeros(3);
	fputc(0x91,fw);
	writezeros(3);
	fputc(0x49,fw);
	writezeros(3);
	fputc(0x01,fw);
	writezeros(1);
	fputc(0x18,fw);
	writezeros(9);
	fputc(0x12,fw);
	fputc(0x0b,fw);
	writezeros(2);
	fputc(0x12,fw);
	fputc(0x0b,fw);
	writezeros(10);
}

//-----------------------------------------------------------------------------
// Name: writezeros()
// Desc: Write zeros into the file
//-----------------------------------------------------------------------------

void FontExtractor::writezeros(int numzero){
	for(int a = 0; a < numzero; a++){
		fputc(0x00,fw);
	}
}