#include <stdio.h>
#include "globals.h"
#include "graphics.h"
#include "maps.h"
#include "tile.h"

//-----------------------------------------------------------------------------
// Name: getmapcoordinates
// Desc: Get the map coordinates
//-----------------------------------------------------------------------------

char* Graphics::getmapcoordinates(){
	return mapcoordinates;
}

//-----------------------------------------------------------------------------
// Name: getWorldTile
// Desc: Get the tile for the world
//-----------------------------------------------------------------------------

char* Graphics::getWorldTile(int tilenum){
	return worldtiles[tilenum];
}

//-----------------------------------------------------------------------------
// Name: createmapcoordinates
// Desc: Create the new map coordinates
//-----------------------------------------------------------------------------

HRESULT Graphics::createmapcoordinates(int currentxpos, int currentypos){
	HRESULT hr;

	SAFE_DELETE( MapCoordinates );
	setmapcoordinates(currentxpos,currentypos);
	if( FAILED( hr = DDisplay->CreateSurfaceFromText( &MapCoordinates, NULL, mapcoordinates, 
		RGB(255,255,255), RGB(115,77,34) ) ) ){
		return hr;
	}
	if( FAILED( hr = MapCoordinates->SetColorKey( RGB(255,255,255) ) ) ){
		return hr;
	}
	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: InitPictures
// Desc: Load the pictures into memory
//-----------------------------------------------------------------------------

HRESULT Graphics::InitPictures(){
	HRESULT hr;
	int animnum = 0;
	int width = 0;
	bool animate1 = false;
	char picture[_MAX_PATH];
	// Create a surface, and draw a bitmap resource on it.  
	
	if( FAILED( hr = DDisplay->CreateSurfaceFromBitmap( &U5Logo, ".\\pictures\\menu\\logo.bmp", 
		116, 32 ) ) ){
        return hr;
	}
	
	if( FAILED( hr = U5Logo->SetColorKey( 0 ) ) ){
        return hr;
	}
	
	if( FAILED( hr = DDisplay->CreateSurfaceFromBitmap( &HighlightedTile, ".\\pictures\\menu\\badtile.bmp", 
		32, 32 ) ) ){
        return hr;
	}
	
	if( FAILED( hr = HighlightedTile->SetColorKey( 0 ) ) ){
        return hr;
	}
	
	if( FAILED( hr = DDisplay->CreateSurfaceFromBitmap( &Menu1, ".\\pictures\\menu\\menu1.bmp", 
		384, 48 ) ) ){
        return hr;
	}
	
	if( FAILED( hr = DDisplay->CreateSurfaceFromBitmap( &Menu2, ".\\pictures\\menu\\menu2.bmp", 
		256, 416 ) ) ){
        return hr;
	}
	
	if( FAILED( hr = DDisplay->CreateSurfaceFromBitmap( &BGL1, ".\\pictures\\menu\\bgl1.bmp", 
		16, 352 ) ) ){
        return hr;
	}
	
	if( FAILED( hr = DDisplay->CreateSurfaceFromBitmap( &BGR1, ".\\pictures\\menu\\bgr1.bmp", 
		16, 352 ) ) ){
        return hr;
	}
	
	if( FAILED( hr = DDisplay->CreateSurfaceFromBitmap( &BGB1, ".\\pictures\\menu\\bgb1.bmp", 
		384, 16 ) ) ){
        return hr;
	}
	
	for(int a = 0; a < 256; a++){
		strcpy( picture, ".\\pictures\\tiles\\" );
		strcat( picture, worldtiles[a] );
		
		if(animnum < 66){
			if(a == tileanim[animnum]){
				width = 128;
				animnum++;
				animate1 = true;
			}
			else{
				width = 32;
				animate1 = false;
			}
		}
		else{
			width = 32;
			animate1 = false;
		}
		if( FAILED( hr = DDisplay->CreateSurfaceFromBitmap( &WorldTiles[a],
			picture, width, 32 ) ) ){
			return hr;
		}
		WorldTiles[a]->setAnimate(animate1);
	}
	
	// Create a surface, and draw text to it.  
	/*	if( FAILED( hr = MapCoordinates->SetColorKey( 0 ) ) ){
	return hr;
}*/
	
	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: RestoreSurfaces()
// Desc: Restore all the surfaces, and redraw the sprite surfaces.
//-----------------------------------------------------------------------------

HRESULT Graphics::RestoreSurfaces(){
    HRESULT hr;
	char picture[_MAX_PATH];
	
	// No need to re-create the surface, just re-draw it.

	if( FAILED( hr = U5Logo->DrawBitmap() ) ){
		return hr;
	}

	if( FAILED( hr = Menu1->DrawBitmap() ) ){
		return hr;
	}

	if( FAILED( hr = Menu2->DrawBitmap() ) ){
		return hr;
	}

	if( FAILED( hr = BGB1->DrawBitmap() ) ){
		return hr;
	}

	if( FAILED( hr = BGR1->DrawBitmap() ) ){
		return hr;
	}

	if( FAILED( hr = BGL1->DrawBitmap() ) ){
		return hr;
	}

	if( FAILED( hr = HighlightedTile->DrawBitmap() ) ){
		return hr;
	}

	for(int a = 0; a < 256; a++){
		strcpy( picture, ".\\pictures\\" );
		strcat( picture,  getWorldTile(a));
		if( FAILED( hr = WorldTiles[a]->DrawBitmap() ) ){
			return hr;
		}
	}

	// No need to re-create the surface, just re-draw it.
    if( FAILED( hr = MapCoordinates->DrawText( NULL, getmapcoordinates(), 
		0, 0, RGB(0,0,0), RGB(255, 255, 0) ) ) ){
        return hr;
	}
	
    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: getViewTile
// Desc: Return whether the tile is opaque, visible, or a window
//-----------------------------------------------------------------------------

int Graphics::getViewTile(int tilenum){
	for(int a = 0; a < 24; a++){
		if(viewtiles1[a] == tilenum){
			return 1;
		}
	}
	for(a = 0; a < 4; a++){
		if(viewtiles2[a] == tilenum){
			return 2;
		}
	}
	return 0;
}

//-----------------------------------------------------------------------------
// Name: changeanimrect
// Desc: Animate the tile
//-----------------------------------------------------------------------------

void Graphics::changeanimrect(){
	animrect.left = (animrect.left + 32)%128;
	animrect.right = animrect.left + 32;
}

//-----------------------------------------------------------------------------
// Name: drawedittiles
// Desc: Draw the tiles for the editor
//-----------------------------------------------------------------------------

void Graphics::drawedittiles(){
	int curtile;
	RECT rc;

	for(int b = 0; b < 4;b++){
		for(int a = 0; a < 8;a++){
			curtile = currenttileselection * 32 + b*8 + a;
			if(WorldTiles[curtile]->IsAnimate() == true){
				rc = animrect;
			}
			else{
				rc = staticrect;
			}
			DDisplay->Blt( a*32+384,b*32+272, WorldTiles[curtile], &rc );
		}
	}

	if(WorldTiles[currentedittile]->IsAnimate() == true){
		rc = animrect;
	}
	else{
		rc = staticrect;
	}
	DDisplay->Blt( 336,8, WorldTiles[currentedittile], &rc );

	DDisplay->Blt( 16, 16, MapCoordinates, NULL );
}

//-----------------------------------------------------------------------------
// Name: drawmap
// Desc: Draw the map on the screen
//-----------------------------------------------------------------------------

void Graphics::drawmap(){
	RECT rc;
	bool hidetile;
	if(animate == true){
		loaddisplaymap();
		setvisibility();
	}
	for(int b = 1; b < 12;b++){
		for(int a = 1; a < 12;a++){
			if(WorldTiles[(displaymap[a][b].tilenum)]->IsAnimate() == true){
				rc = animrect;
			}
			else{
				rc = staticrect;
			}
			if(lineofsight == true){
				hidetile = displaymap[a][b].isdark;
			}
			else{
				hidetile = false;
			}
			if(hidetile == false){
				DDisplay->Blt( (a-1)*32+16,(b-1)*32+48, WorldTiles[(displaymap[a][b].tilenum)], &rc );
			}
			if(highlighttiles == true){
				if(displaymap[a][b].highlighted == true){
					DDisplay->Blt( (a-1)*32+16,(b-1)*32+48, HighlightedTile, &staticrect );
				}
			}
		}
	}
}

//-----------------------------------------------------------------------------
// Name: drawmenu
// Desc: Draw the menu
//-----------------------------------------------------------------------------

void Graphics::drawmenu(){
	DDisplay->Blt( 0,48, BGL1, NULL );
	DDisplay->Blt( 368,48, BGR1, NULL );
	DDisplay->Blt( 0,400, BGB1, NULL );
	DDisplay->Blt( 0,0, Menu1, NULL );
	DDisplay->Blt( 384,0, Menu2, NULL );
	DDisplay->Blt( 134,9, U5Logo, NULL );
}

//-----------------------------------------------------------------------------
// Name: finish
// Desc: Clean up Class Graphics
//-----------------------------------------------------------------------------

void Graphics::finish(){
	SAFE_DELETE( U5Logo );
	SAFE_DELETE( Menu1 );
	SAFE_DELETE( Menu2 );
	SAFE_DELETE( BGL1 );
	SAFE_DELETE( BGR1 );
	SAFE_DELETE( BGB1 );
	SAFE_DELETE( HighlightedTile );
	for(int a = 0; a < 256; a++ ){
		SAFE_DELETE( WorldTiles[a] );
	}
	SAFE_DELETE( MapCoordinates );
}

//-----------------------------------------------------------------------------
// Name: initRect
// Desc: Initialize the graphic's rectangles
//-----------------------------------------------------------------------------

void Graphics::initRect(){
	animrect.bottom = 32;
	animrect.top = 0;
	animrect.right = 32;
	animrect.left = 0;
	staticrect.bottom = 32;
	staticrect.top = 0;
	staticrect.right = 32;
	staticrect.left = 0;
}

//-----------------------------------------------------------------------------
// Name: loaddisplaymap
// Desc: Load the display map
//-----------------------------------------------------------------------------

void Graphics::loaddisplaymap(){
	int x;
	int y;
	highlighted = false;
	for(int b = 0; b < 13; b++){
		for(int a = 0; a < 13; a++){
			x = a - 6 + currentxpos;
			y = b - 6 + currentypos;
			if(wrap == true){
				if(x < 0){
					x = maxX + x + 1;
				}
				else if(x > maxX){
					x = x - maxX - 1;
				}
				if(y < 0){
					y = maxY + y + 1;
				}
				else if(y > maxY){
					y = y - maxY - 1;
				}
			}
			else{
				if(x < 0 || x > maxX || y < 0 || y > maxY){
					x = maxX;
					y = maxY;
					highlighted = true;
				}
				else{
					highlighted = false;
				}
			}
			displaymap[a][b].tilenum = map[x][y].tilenum;
			displaymap[a][b].highlighted = map[x][y].highlighted;
			if(highlighted == true){
				displaymap[a][b].highlighted = true;
			}
		}
	}
}

//-----------------------------------------------------------------------------
// Name: setmapcoordinates
// Desc: Set the new map coordinates
//-----------------------------------------------------------------------------

void Graphics::setmapcoordinates(int x, int y){
	sprintf(mapcoordinates,"%d, %d",x,y);
}

//-----------------------------------------------------------------------------
// Name: setvisibility
// Desc: Set the visibility for the display map
//-----------------------------------------------------------------------------

void Graphics::setvisibility(){
	for(int b = 0; b < 13; b++){
		for(int a = 0; a < 13; a++){
			displaymap[a][b].visited = false;
			displaymap[a][b].visible = false;
			displaymap[a][b].isdark = true;
			displaymap[a][b].visibility = getViewTile(displaymap[a][b].tilenum);
		}
	}
	displaymap[6][6].visible = true;
	displaymap[6][6].visited = true;
	if(displaymap[5][6].visibility == 2){
		displaymap[5][6].visible = true;
	}
	if(displaymap[6][5].visibility == 2){
		displaymap[6][5].visible = true;
	}
	if(displaymap[7][6].visibility == 2){
		displaymap[7][6].visible = true;
	}
	if(displaymap[6][7].visibility == 2){
		displaymap[6][7].visible = true;
	}
	setvisibility1(6,6);
	setvisibility2();
}

//-----------------------------------------------------------------------------
// Name: setvisibility1
// Desc: Set the visibility for the display map recursively
//-----------------------------------------------------------------------------

void Graphics::setvisibility1(int x, int y){
	displaymap[x][y].visited = true;
	if(displaymap[x][y].visible == false){
		return;
	}
	if(x > 1 && y > 1){
		if(displaymap[x-1][y-1].visited == false){
			if(displaymap[x-1][y-1].visibility == 0){
				displaymap[x-1][y-1].visible = true;
			}
			setvisibility1(x-1,y-1);
		}
	}
	if(y > 1){
		if(displaymap[x][y-1].visited == false){
			if(displaymap[x][y-1].visibility == 0){
				displaymap[x][y-1].visible = true;
			}
			setvisibility1(x,y-1);
		}
	}
	if(x < 11 && y > 1){
		if(displaymap[x+1][y-1].visited == false){
			if(displaymap[x+1][y-1].visibility == 0){
				displaymap[x+1][y-1].visible = true;
			}
			setvisibility1(x+1,y-1);
		}
	}
	if(x > 1){
		if(displaymap[x-1][y].visited == false){
			if(displaymap[x-1][y].visibility == 0){
				displaymap[x-1][y].visible = true;
			}
			setvisibility1(x-1,y);
		}
	}
	if(x < 11){
		if(displaymap[x+1][y].visited == false){
			if(displaymap[x+1][y].visibility == 0){
				displaymap[x+1][y].visible = true;
			}
			setvisibility1(x+1,y);
		}
	}
	if(x > 1 && y < 11){
		if(displaymap[x-1][y+1].visited == false){
			if(displaymap[x-1][y+1].visibility == 0){
				displaymap[x-1][y+1].visible = true;
			}
			setvisibility1(x-1,y+1);
		}
	}
	if(y < 11){
		if(displaymap[x][y+1].visited == false){
			if(displaymap[x][y+1].visibility == 0){
				displaymap[x][y+1].visible = true;
			}
			setvisibility1(x,y+1);
		}
	}
	if(x < 11 && y < 11){
		if(displaymap[x+1][y+1].visited == false){
			if(displaymap[x+1][y+1].visibility == 0){
				displaymap[x+1][y+1].visible = true;
			}
			setvisibility1(x+1,y+1);
		}
	}
}

//-----------------------------------------------------------------------------
// Name: setvisibility2
// Desc: Set the visibility for tiles surrounding the center
//-----------------------------------------------------------------------------

void Graphics::setvisibility2(){
	for(int y = 1; y < 12; y++){
		for(int x = 1; x < 12; x++){
			if(displaymap[x][y].visible == true){
				displaymap[x][y].isdark = false;
			}
			else{
				if(displaymap[x-1][y-1].visible == true){
					displaymap[x][y].isdark = false;
				}
				if(displaymap[x][y-1].visible == true){
					displaymap[x][y].isdark = false;
				}
				if(displaymap[x+1][y-1].visible == true){
					displaymap[x][y].isdark = false;
				}
				if(displaymap[x-1][y].visible == true){
					displaymap[x][y].isdark = false;
				}
				if(displaymap[x+1][y].visible == true){
					displaymap[x][y].isdark = false;
				}
				if(displaymap[x-1][y+1].visible == true){
					displaymap[x][y].isdark = false;
				}
				if(displaymap[x][y+1].visible == true){
					displaymap[x][y].isdark = false;
				}
				if(displaymap[x+1][y+1].visible == true){
					displaymap[x][y].isdark = false;
				}
			}
		}
	}
}