#define STRICT
#include <stdio.h>
#include <ddraw.h>
#include <windowsx.h>
#include <mmsystem.h>
#include "resource.h"
#include "pictures.h"
#include "menu.h"
#include "DDInit.h"
#include "miscfunc.h"
#include "globals.h"
#include "dialogs.h"
#include "fonts.h"

//-----------------------------------------------------------------------------
//	Global Variables
//-----------------------------------------------------------------------------

static BOOL	g_bActive	   = FALSE;
static FontExtractor* fontextract;
static MiscFunc* miscfunc;
static DDInit* ddinit;
static LZW2BMP *lzw2bmp;
static BMP2LZW *bmp2lzw;

static int xpos;
static int ypos;

static char position[100];

static HINSTANCE hInstance;

//-----------------------------------------------------------------------------
// Name: MainWndProc()
// Desc: The main window procedure
//-----------------------------------------------------------------------------
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	int wmId;
	int wmEvent;

	switch (msg){
	case WM_CREATE:
		makeMenu(hWnd);
		break;
	case WM_COMMAND:
		wmId	= LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		// Parse the menu selections:
		switch (wmId){
		case ID_DIRECTORY_OPEN:
			if(!OnOpenDirectory(hWnd, miscfunc)){			// try opening the file
				return 0;
			}
			break;
		case ID_FILE_SAVE:
			switch(currentmode){
			case EDITMAPS:
				miscfunc->saveu5directory(hWnd);
			default:
				break;
			}
			break;
			case ID_FILE_EXIT:
			SendMessage(hWnd,WM_DESTROY,0,0);	// quit the program
			break;
		case ID_HELP_ABOUT:
			DialogBox(hInstance, (LPCTSTR)IDD_ABOUT, hWnd, (DLGPROC)About);
			break;

		case ID_GOTO_POS:
			DialogBox(hInstance, (LPCTSTR)IDD_GOTOPOS, hWnd, (DLGPROC)GotoPos);
			createmapcoordinates();
			break;
		////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////

		case ID_MAP_BRIT:
		case ID_MAP_UNDER:
		case ID_MAP_BRITAIN:
		case ID_MAP_JHELOM:
		case ID_MAP_MINOC:
		case ID_MAP_MOONGLOW:
		case ID_MAP_NEWMAGINCIA:
		case ID_MAP_SKARABRAE:
		case ID_MAP_TRINSIC:
		case ID_MAP_YEW:
		case ID_MAP_CASTLEBLACKTHORN:
		case ID_MAP_CASTLEBRITISH:
		case ID_MAP_EMPATHABBEY:
		case ID_MAP_LYCAEUM:
		case ID_MAP_SERPENTSHOLD:
		case ID_MAP_GRENDEL:
		case ID_MAP_IOLO:
		case ID_MAP_SINVRAAL:
		case ID_MAP_SUTEK:
		case ID_MAP_BUCSDEN:
		case ID_MAP_COVE:
		case ID_MAP_EASTBRITAINNY:
		case ID_MAP_NORTHBRITAINNY:
		case ID_MAP_PAWS:
		case ID_MAP_WESTBRITAINNY:
		case ID_MAP_ARARAT:
		case ID_MAP_BORDERMARCH:
		case ID_MAP_FARTHING:
		case ID_MAP_STONEKEEP:
		case ID_MAP_WINDEMERE:
		case ID_MAP_FOGSBANE:
		case ID_MAP_GREYHAVEN:
		case ID_MAP_STORMCROW:
		case ID_MAP_WAVEGUIDE:
			miscfunc->changemap(hWnd, wmId);
			break;

		case ID_LEVEL1:
			miscfunc->changelevel(0);
			checklevelmenu(hWnd, 1);
			break;
		case ID_LEVEL2:
			miscfunc->changelevel(1);
			checklevelmenu(hWnd, 2);
			break;
		case ID_LEVEL3:
			miscfunc->changelevel(2);
			checklevelmenu(hWnd, 3);
			break;
		case ID_LEVEL4:
			miscfunc->changelevel(3);
			checklevelmenu(hWnd, 4);
			break;
		case ID_LEVEL5:
			miscfunc->changelevel(4);
			checklevelmenu(hWnd, 5);
			break;

		case ID_LINEOFSIGHT:
			changelineofsight();
			break;

		case ID_HIGHLIGHTTILES:
			if(highlighttiles == true){
				checkmapoptionsmenu(hWnd, UNCHECKHIGHLIGHTTILE);
			}
			else{
				checkmapoptionsmenu(hWnd, CHECKHIGHLIGHTTILE);
			}
			break;

		case ID_CHANGEFONT2BITMAP:
			fontextract = new FontExtractor();
			fontextract->changefonttobmp("ibm",path);
			fontextract->changefonttobmp("runes",path);
			SAFE_DELETE(fontextract);
			break;

		case ID_CHANGEBITMAP2FONT:
			fontextract = new FontExtractor();
			fontextract->changebmptofont("ibm",path);
			fontextract->changebmptofont("runes",path);
			SAFE_DELETE(fontextract);
			break;
		case ID_EXTRACTGRAPHICS:
			lzw2bmp = new LZW2BMP();
			SAFE_DELETE(lzw2bmp);
			break;
		case ID_INSERTGRAPHICS:
			bmp2lzw = new BMP2LZW();
			SAFE_DELETE(bmp2lzw);
			break;
		////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////
		default:
			break;
		}
		case WM_PAINT:
            // Update the screen if we need to refresh. This case occurs 
            // when in windowed mode and the window is behind others.
            // The app will not be active, but it will be visible.
            if( DDisplay ){
                // Display the new position of the sprite
                if( ddinit->DisplayFrame() == DDERR_SURFACELOST )
                {
                    // If the surfaces were lost, then restore and try again
                    ddinit->RestoreSurfaces();
                    ddinit->DisplayFrame();
                }
            }
			lbuttondown=false;
            break; // Continue with default processing to validate the region

		case WM_GETMINMAXINFO:
            {
                // Don't allow resizing in windowed mode.  
                // Fix the size of the window to 640x480 (client size)
                MINMAXINFO* pMinMax = (MINMAXINFO*) lParam;

                DWORD dwFrameWidth    = GetSystemMetrics( SM_CXSIZEFRAME );
                DWORD dwFrameHeight   = GetSystemMetrics( SM_CYSIZEFRAME );
                DWORD dwMenuHeight    = GetSystemMetrics( SM_CYMENU );
                DWORD dwCaptionHeight = GetSystemMetrics( SM_CYCAPTION );

                pMinMax->ptMinTrackSize.x = WINDOW_WIDTH  + dwFrameWidth * 2;
                pMinMax->ptMinTrackSize.y = WINDOW_HEIGHT + dwFrameHeight * 2 + 
                                            dwMenuHeight + dwCaptionHeight;

                pMinMax->ptMaxTrackSize.x = pMinMax->ptMinTrackSize.x;
                pMinMax->ptMaxTrackSize.y = pMinMax->ptMinTrackSize.y;
            }
            return 0L;

        case WM_MOVE:
			if( DDisplay ){
		        DDisplay->UpdateBounds();
			}
			lbuttondown=false;
            return 0L;

        case WM_SIZE:
            // Check to see if we are losing our window...
            if( SIZE_MAXHIDE==wParam || SIZE_MINIMIZED==wParam )
                g_bActive = FALSE;
            else
                g_bActive = TRUE;

			if( DDisplay ){
		        DDisplay->UpdateBounds();
			}
			lbuttondown=false;
            break;
            
		case WM_KEYDOWN:
			animate = true;
			switch(currentmode){
			case EDITMAPS:
				switch (wParam){
				case VK_LEFT:
					currentxpos--;
					if(currentxpos < 0){
						if(wrap == true){
							currentxpos = maxX;
						}
						else{
							currentxpos = 0;
						}
					}
					createmapcoordinates();
					if(lbuttondown == true){
						miscfunc->getPicture(xpos,ypos);
					}
					break;
				case VK_RIGHT:
					currentxpos++;
					if(currentxpos > maxX){
						if(wrap == true){
							currentxpos = 0;
						}
						else{
							currentxpos = maxX;
						}
					}
					createmapcoordinates();
					if(lbuttondown == true){
						miscfunc->getPicture(xpos,ypos);
					}
					break;
				case VK_UP:
					currentypos--;
					if(currentypos < 0){
						if(wrap == true){
							currentypos = maxY;
						}
						else{
							currentypos = 0;
						}
					}
					createmapcoordinates();
					if(lbuttondown == true){
						miscfunc->getPicture(xpos,ypos);
					}
					break;
				case VK_DOWN:
					currentypos++;
					if(currentypos > maxY){
						if(wrap == true){
							currentypos = 0;
						}
						else{
							currentypos = maxY;
						}
					}
					createmapcoordinates();
					if(lbuttondown == true){
						miscfunc->getPicture(xpos,ypos);
					}
					break;
				case VK_PRIOR:
					if(currenttileselection > 0){
						currenttileselection--;
					}
					else{
						currenttileselection = 7;
					}
					break;
				case VK_NEXT:
					if(currenttileselection < 7){
						currenttileselection++;
					}
					else{
						currenttileselection = 0;
					}
					break;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			
			case WM_LBUTTONUP:
				animate = true;
				lbuttondown=false;
				switch(currentmode){
				case EDITMAPS:
					break;
				default:
					break;
				}
				break;
			case WM_LBUTTONDOWN:
				animate = true;
				xpos = GET_X_LPARAM(lParam); 
				ypos = GET_Y_LPARAM(lParam);
				sprintf(position,"%d,%d",xpos,ypos);
				lbuttondown=true;
				switch(currentmode){
				case EDITMAPS:
					miscfunc->getPicture(xpos,ypos);
					break;
				default:
					break;
				}
				break;
			case WM_MOUSEMOVE:
				switch(currentmode){
				case EDITMAPS:
					if(lbuttondown == true){
						xpos = GET_X_LPARAM(lParam); 
						ypos = GET_Y_LPARAM(lParam); 
						animate = true;
						miscfunc->getPicture(xpos,ypos);
					}
					break;
				default:
					break;
				}
				break;
				
			case WM_DESTROY:
				// Cleanup and close the app
				SAFE_DELETE( miscfunc );
				SAFE_DELETE( ddinit );
				PostQuitMessage( 0 );
				return 0L;
    }
	
	return DefWindowProc(hWnd, msg, wParam, lParam);
}


//-----------------------------------------------------------------------------
// Name: WinInit()
// Desc: Init the window
//-----------------------------------------------------------------------------
HRESULT WinInit(HWND* hWnd, HINSTANCE hInst, int nCmdShow){
	WNDCLASSEX wc;
	// Register the Window Class
	wc.cbSize		 = sizeof(wc);
	wc.lpszClassName = TEXT("WindowedMode");
	wc.lpfnWndProc	 = MainWndProc;
	wc.style		 = CS_VREDRAW | CS_HREDRAW;
	wc.hInstance	 = hInst;
	wc.hIcon		 = LoadIcon( hInst, MAKEINTRESOURCE(IDI_ICON1) );
	wc.hIconSm		 = LoadIcon( hInst, MAKEINTRESOURCE(IDI_ICON1) );
	wc.hCursor		 = LoadCursor( NULL, IDC_ARROW );
	wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
	wc.lpszMenuName  = 0;
	wc.cbClsExtra	 = 0;
	wc.cbWndExtra	 = 0;
	
	if( RegisterClassEx( &wc ) == 0 )
		return E_FAIL;
	
	// Calculate the proper size for the window given a client of 640x480
	DWORD dwFrameWidth	  = GetSystemMetrics( SM_CXSIZEFRAME );
	DWORD dwFrameHeight   = GetSystemMetrics( SM_CYSIZEFRAME );
	DWORD dwMenuHeight	  = GetSystemMetrics( SM_CYMENU );
	DWORD dwCaptionHeight = GetSystemMetrics( SM_CYCAPTION );
	DWORD dwWindowWidth   = WINDOW_WIDTH  + dwFrameWidth * 2;
	DWORD dwWindowHeight  = WINDOW_HEIGHT + dwFrameHeight * 2 + 
		dwMenuHeight + dwCaptionHeight;
	
	// Create and show the main window
	DWORD dwStyle = WS_OVERLAPPEDWINDOW & ~WS_MAXIMIZEBOX;
	*hWnd = CreateWindowEx( 0, TEXT("WindowedMode"), TEXT("Ultima V Editor 0.02 Alpha - Nytegard"),
		dwStyle, CW_USEDEFAULT, CW_USEDEFAULT,
		dwWindowWidth, dwWindowHeight, NULL, NULL, hInst, NULL );
	if( hWnd == NULL )
		return E_FAIL;
	
	ShowWindow(*hWnd, nCmdShow );
	UpdateWindow(*hWnd );

	hInstance = hInst;

	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything and calls
//		 UpdateFrame() when idle from the message pump.
//-----------------------------------------------------------------------------
int APIENTRY WinMain( HINSTANCE hInst, HINSTANCE hPrevInst,
					 LPSTR pCmdLine, int nCmdShow ){
	MSG		msg;
	HWND	hWnd;

	miscfunc = new MiscFunc();
	ddinit = new DDInit();
	
	if( FAILED( WinInit(&hWnd, hInst, nCmdShow) ) )
		return FALSE;
	
	if( FAILED( ddinit->InitDirectDraw(hWnd) ) ){
        MessageBox( hWnd, TEXT("DirectDraw init failed. ")
			TEXT("Now exiting. "), TEXT("Ultima V Editor"), 
			MB_ICONERROR | MB_OK );
        return FALSE;
	}
	while( TRUE ){
		// Look for messages, if none are found then 
		// update the state and display it
		if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) ){
			if( 0 == GetMessage(&msg, NULL, 0, 0 ) ){
				// WM_QUIT was posted, so exit
				return (int)msg.wParam;
			}
			
			// Translate and dispatch the message
			TranslateMessage( &msg ); 
			DispatchMessage( &msg );
		}
		else{
			if( g_bActive )
            {
                // Move the sprites, blt them to the back buffer, then 
                // flip or blt the back buffer to the primary buffer
                if( FAILED( ddinit->ProcessNextFrame(hWnd) ) ){
                    SAFE_DELETE( DDisplay );

                    MessageBox( hWnd, TEXT("Display failed. ")
                                TEXT("The program will now exit. "), TEXT("Ultima V Editor"), 
                                MB_ICONERROR | MB_OK );
                    return FALSE;
                }
            }
			else{
				// Make sure we go to sleep if we have nothing else to do
				WaitMessage();
			}
		}
	}
}
